# Using Ubuntu LTS

Unlike the *tutorials*, the development environments cannot be entirely encapsulated in Docker images.  For almost all development purposes, you will also need a "real" or virtual Ubuntu Linux system, with ROS installed, for the actual development of your own software.

The supported operating system versions are **Ubuntu 20.04 and 18.04 LTS** (Long Term Support).  Other versions, such as the more recent 20.10, are not supported by ROS or by the software in the development environment.  If you are installing an operating system now (as opposed to continuing with an existing installation), we definitely recommend **Ubuntu 20.04**, which you can install either directly on your hardware (recommended) or in a virtual machine.

## Terminology

- **Main Operating System**:  The operating system that runs directly on your hardware.

- **Development System**:  A *real computer* or *virtual machine* running *Ubuntu 18.04 or 20.04*, where you intend to do software development.

## Alternatives

The main ways of developing software using the Development Environment are the following:

- **Use Ubuntu 20.04 as the main operating system** on your computer, and do not upgrade until the release of (a) Ubuntu 22.04 LTS, (b) a new ROS version for Ubuntu 22.04 LTS, and (c) a new version of the development environments adapted to these new systems.  This is the main alternative for many developers as the long term support versions provide a stable development system that minimizes the risk of incompatibility.

- Use **Ubuntu 20.04 in a virtual machine running on another version of Linux**, such as a newer version of Ubuntu.  This is the best supported alternative to using Ubuntu 20.04 directly on your computer.  However, there can be issues with 3D graphics, such as slowdowns due to virtualization or in some cases incompatibilities between drivers.  We have successfully used Ubuntu 20.04 in a virtual machine on 20.10, but have not tested other variations. 

- Use **Ubuntu 20.04 in a virtual machine running on a Windows machine**.  This will lead to some issues in terms of using 3D graphics, but 2D graphics should work.  There are some [initial instructions](https://gitlab.liu.se/lrs/symbicloud_docker_images/-/blob/master/doc/itn.md) for this, and we will try to help out in case of problems, but cannot guarantee that absolutely all aspects of a development environment will work as expected.

- Use **Ubuntu 20.04 in a virtual machine running on MacOS**.  This will lead to serious performance issues in terms of using 3D graphics.  2D graphics may work better.

In certain limited cases, the development environment docker containers can run directly on Windows or MacOS.  This entails communicating with the surrounding environment through for example MQTT, with very limited possibilities of using graphics.

## Installing a Virtual Machine running Ubuntu 18.04 or 20.04

*These instructions are quite terse, as there exist plenty of descriptions on the web.*

As noted above, if you do not run Ubuntu 20.04 (or 18.04) on your machine, you need to install a virtual machine.  You may use software such as [VirtualBox](https://www.virtualbox.org/), [Parallels](https://www.parallels.com/se/) or [VMWare Player](https://www.vmware.com/se/products/workstation-player.html) for this purpose.  We have mainly tested VirtualBox, as it is free.

For VirtualBox:

- Install VirtualBox
- Download [installation media](https://ubuntu.com/download/desktop) for Ubuntu 20.04.x LTS 
- Start VirtualBox and create a new virtual machine
- Install Ubuntu from the installation media that you downloaded
- If and when Virtual box asks about the network:  You want a *bridged* network.
- You need to do the following in the *virtual* machine.  One reason is that you need to build the [VirtualBox Guest Addition](https://www.virtualbox.org/manual/ch04.html) modules that allow the "virtual Ubuntu" to communicate with the virtual machine software more efficiently, speed up graphics, allow the use of shared folders, and so on. 

  ```bash
  sudo apt install gcc perl make   # to build guest addition kernel modules
  sudo apt install net-tools openssh-server
  ```

- You also need to actually *build and install* the [VirtualBox Guest Additions](https://help.ubuntu.com/community/VirtualBox/GuestAdditions).

##### Contact information

To ask questions about the development environment, contact `lrs-devenv@groups.liu.se`.